define([
    "dojo/_base/declare",
    "thornberry/app/_Module",
    "thornberry/app/Component/LazyLoadView",
    "thornberry/app/Component/BillingManagement/Dashboard",
    "thornberry/app/Component/BillingManagement/UnbilledClaimsView",
    "thornberry/app/Component/BillingManagement/BilledClaimsView",
    "thornberry/app/Component/BillingManagement/ClaimFilesView",
    "thornberry/app/Component/BillingManagement/ClaimView",
    "thornberry/app/Service/BillingManagement",
    "dojo/dom-attr",
    "dojo/aspect",
    "thornberry/app/utils"
],function(
    declare,
    _Module,
    LazyLoadView,
    Dashboard,
    UnbilledClaimsView,
    BilledClaimsView,
    ClaimFilesView,
    ClaimView,
    service,
    domAttr,
    aspect,
    utils
) {
    onOpenClaim = null; // global method for use in 

    return declare(_Module,{
        title: "Billing Management",

        initPage: function() {
            var t = this;
            var page = t.inherited(arguments);

            onOpenClaim = function(id) {
                var view = new ClaimView({
                    props: {
                        viewOnly: true,
                        mode: "view",
                        objectId: id
                    }
                });
                page.pushViewContainers(view);
            };

            var onSearchClaim = function(claimId) {
                page.setLoadingPromise(
                    service.findClaimId(claimId).then(onOpenClaim,utils.showMessage)
                );
            }

            aspect.before(page,"refreshRendering",function() {
                page.props.searchButtonShown = true;
                page.props.onSearch = onSearchClaim;
            });
            page.searchInput.set("placeHolder","Enter Claim ID");
            domAttr.set(page.searchButtonNode,"title","Find Claim by ID");

            return page;
        },

        initViews: function(page) {
            var viewMap = {}
            var props = {
                props: {
                    onSetViewFilters: function(viewName,filters) {
                        var view = viewMap[viewName];
                        if (view) {
                            view.load().set({
                                props: {
                                    filters: filters
                                }
                            })
                        }
                    },
                    onUpdateData: function() {
                        for (var viewName in viewMap) {
                            viewMap[viewName].instance.refreshData();
                        }
                    }
                }
            };

            viewMap["Dashboard"]        = new LazyLoadView({ viewClass: Dashboard,              title: "Dashboard",             props: props });
            viewMap["UnbilledClaims"]   = new LazyLoadView({ viewClass: UnbilledClaimsView,     title: "Unbilled Claims",       props: props,   preload: true });
            viewMap["BilledClaims"]     = new LazyLoadView({ viewClass: BilledClaimsView,       title: "Billed Claims",         props: props,   preload: true });
            viewMap["ClaimFiles"]       = new LazyLoadView({ viewClass: ClaimFilesView,         title: "Claim Files",           props: props,   preload: true });
           
            page.pushViewContainers([
                viewMap["Dashboard"],
                viewMap["UnbilledClaims"],
                viewMap["BilledClaims"],
                viewMap["ClaimFiles"]
            ]);
        }
    });
});