define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"thornberry/app/Component/_EditorComponent",
	"thornberry/app/Component/_DataFormMixin",
	"thornberry/app/Component/_TableFieldEditorMixin",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!./Templates/AccountEditor.html",
	"dojo/text!./Templates/PatientSection.html",
	"dojo/text!./Templates/ClaimSection.html",
	"dojo/text!./Templates/DiagnosisSection.html",
	"thornberry/object/TableManager",
	"dojo/dom-style",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/when",
	"gridx/Grid",
	"gridx/core/model/cache/Sync",
	"gridx/modules/CellWidget",
	"dojo/store/Memory",
	"thornberry/app/utils",
	"dijit/form/ValidationTextBox",
	"dijit/form/Textarea",
	"dijit/form/Button"
],function(
	declare,
	lang,
	arrayUtils,
	_EditorComponent,
	_DataFormMixin,
	_TableFieldEditorMixin,
	_TemplatedMixin,
	_WidgetsInTemplateMixin,
	template,
	patientSectionTemplate,
	claimSectionTemplate,
	diagnosisSectionTemplate,
	TableManager,
	domStyle,
	domClass,
	domConstruct,
	when,
	Grid,
	Cache,
	CellWidget,
	Memory,
	utils
) {
	return declare([_EditorComponent, _DataFormMixin, _TemplatedMixin, _WidgetsInTemplateMixin, _TableFieldEditorMixin], {
		templateString: template,

		postMixInProperties: function() {
			var t = this;
			t.inherited(arguments);

			t.templateString = lang.replace(template,{
				patientSectionTemplate: patientSectionTemplate,
				claimSectionTemplate:   claimSectionTemplate,
				diagnosisSectionTemplate: diagnosisSectionTemplate
			});
		},

		buildRendering: function() {
			var t = this;
			t.inherited(arguments);

			var canEditDecorator = function() {
				return "<div data-dojo-type='dijit/form/Button' data-dojo-attach-point='editButton' data-dojo-props=\"label:'Edit'\"></div>";
			};
			var canEditSetCellValue = function(gridData,storeData,cellWidget) {
				var id = cellWidget.cell.row.id;
				var data = t.chargeGrid.store.get(id);
				var editButton = cellWidget.editButton;
				if (editButton._cnnt) {
					editButton._cnnt.remove();
				}
				if ((data.id === "summary") || (t.props.disabled) || (!data.canEdit)) {
					domStyle.set(editButton.domNode, "display", "none");
				} else {
					domStyle.set(editButton.domNode ,"display", "");

					editButton._cnnt = on(editButton, "click", function() {
						// TODO: open transaction sub-view?
					});
				}
			};

			var transactionsStructure = [
				{ id: "date", field: "date", name: "Date", formatter: utils.formatters.date("date") },
				{ id: "type", field: "type", name: "Type" },
				{ id: "amount", field: "amount", name: "Amount", formatter: utils.formatters.dollars("amount") },
				{ id: "processDate", field: "processDate", name: "Process Date", formatter: utils.formatters.date("processDate") },
				{ id: "checkNumber", field: "checkNumber", name: "Check #" },
				{ id: "canEdit", field: "canEdit", name: " ", widgetsInCell: true, decorator: canEditDecorator, setCellValue: canEditSetCellValue, width: "60px" }
			];
			t.transactionsGrid = new Grid({
				cacheClass: Cache,
				store: new Memory(),
				structure: transactionsStructure,
				autoHeight: true,
				modules: [CellWidget]
			});
			t.transactionsGrid.startup();
			t.transactionGridContainer.appendChild(t.transactionsGrid.domNode);
			var chargeStructure = [
				{ id: "type", field: "type", name: "Type" },
				{ id: "description", field: "description", name: "Description" },
				{ id: "chargeDate", field: "chargeDate", name: "Date", formatter: utils.formatters.date("chargeDate") },
				{ id: "chargeCode", field: "chargeCode", name: "Charge Code" },  
				{ id: "net", field: "net", name: "Net", formatter: utils.formatters.dollars("net"), style: "text-align: right" },
				{ id: "balance", field: "balance", name: "Balance", formatter: utils.formatters.dollars("balance"), style: "text-align: right" },
				{ id: "durQty", field: "durQty", name: "Dur/Qty" },
				{ id: "revenueCode", field: "revenueCode", name: "Revenue Code" },
				{ id: "hcpcsCode", field: "hcpcsCode", name: "HCPCS Code" }
			];
			t.chargeGrid = new Grid({
				cacheClass: Cache,
				store: new Memory(),
				structure: chargeStructure,
				autoHeight: true,
				modules: [CellWidget]
			});
			t.chargeGrid.connect(t.chargeGrid.body, "onAfterRow", function(row) {
				var item = row.grid.model.byId(row.id).item;
				try {
					if (item.id === "summary") {
						domClass.add(row.node(),"bold");
					}
				} catch (e) {console.error(e);}
			});
			t.chargeGrid.startup();
			t.chargeGridContainer.appendChild(t.chargeGrid.domNode);

			var patientPoliciesStructure = [
				{ id: "seq", field: "seq", name: "Seq" },
				{ id: "insurance", field: "insurance", name: "Insurance" },
				{ id: "policyNum", field: "policyNum", name: "Policy #" },
				{ id: "effectiveDate", field: "effectiveDate", name: "Effective", formatter: utils.formatters.date("effectiveDate") },
				{ id: "priorPayments", field: "priorPayments", name: "Prior Payments" },
				{ id: "dcn", field: "dcn", name: "DCN", width: "200px" }
			];
			t.patientPoliciesGrid = new Grid({
				cacheClass: Cache,
				store: new Memory(),
				structure: patientPoliciesStructure,
				autoHeight: true,
				modules: [CellWidget]
			});
			t.patientPoliciesGrid.startup();
 			t.patientPoliciesGridContainer.appendChild(t.patientPoliciesGrid.domNode);

			when(TableManager.get("Tables.ArAccountStatus"),function(table) {
				t.accountStatus.setStore(table);
			});

			var submissionStructure = [
				{ id: "dateTime", field: "dateTime", name: "Date/Time", formatter: function(itemData) { return itemData.date+" "+itemData.time; } },
				{ id: "fileName", field: "fileName", name: "Filename" },
				{ id: "form", field: "form", name: "Form" }
			];
			t.submissionGrid = utils.createGrid(submissionStructure,t.submissionGridContainer);
		},

		refreshRendering: function() {
			var t = this;

			if (t.value !== t.props.value) {
				t.value = t.props.value;
				
				t.set("value",t.props.value);

				t.chargeGrid.setStore(new Memory({data:t.props.value ? t.props.value.charges.charges : []}));
				t.chargeGrid.model.clearCache();
				t.chargeGrid.body.refresh();

				t.patientPoliciesGrid.setStore(new Memory({data:t.props.value ? t.props.value.policies : []}));
				t.patientPoliciesGrid.model.clearCache();
				t.patientPoliciesGrid.body.refresh();

				t.submissionGrid.setStore(new Memory({data:t.props.value ? t.props.value.submissions : []}));
				t.submissionGrid.model.clearCache();
				t.submissionGrid.body.refresh();

				t.transactionsGrid.setStore(new Memory({data:t.props.value ? t.props.value.transactions : []}));
				t.transactionsGrid.model.clearCache();
				t.transactionsGrid.body.refresh();
			} else {
				t.chargeGrid.model.clearCache();
				t.chargeGrid.body.refresh();

				t.patientPoliciesGrid.model.clearCache();
				t.patientPoliciesGrid.body.refresh();

				t.transactionsGrid.model.clearCache();
				t.transactionsGrid.body.refresh();
			}

			var disabled = t.props.disabled;
			var isPrePayment = t.props.value && (t.props.value.type == "Pre-Payment");
			var hasTransactions = t.props.value && t.props.value.transactions.length;
			var hasSubmissions = t.props.value && t.props.value.submissions.length;
			domStyle.set(t.chargeContainer,				"display", isPrePayment || !disabled ? "none" : "");
			domStyle.set(t.patientPoliciesContainer,	"display", isPrePayment || !disabled ? "none" : "");
			domStyle.set(t.submissionContainer,			"display", isPrePayment || !disabled ? "none" : "");
			domStyle.set(t.diagnosisContainer,			"display", isPrePayment || !disabled ? "none" : "");
			domStyle.set(t.transactionsContainer,		"display", isPrePayment || !disabled ? "none" : "");

			domStyle.set(t.noTransactionsMessage,		"display", hasTransactions || isPrePayment ? "none" : "");
			domStyle.set(t.transactionGridContainer,	"display", hasTransactions || isPrePayment ? "" : "none");
			domStyle.set(t.noSubmissionsMessage,		"display", hasSubmissions || isPrePayment ? "none" : "");
			domStyle.set(t.submissionGridContainer,		"display", hasSubmissions || isPrePayment ? "" : "none");
			domStyle.set(t.accountStatusNode,			"display", (disabled ? "" : "none"));
			domStyle.set(t.accountStatusFieldContainer,	"display", (disabled ? "none" : ""));
			domStyle.set(t.dcnNode,						"display", (disabled ? "" : "none"));
			domStyle.set(t.dcnFieldContainer,			"display", (disabled ? "none" : ""));
			domStyle.set(t.followUpDateNode,			"display", (disabled ? "" : "none"));
			domStyle.set(t.followUpDateFieldContainer,	"display", (disabled ? "none" : ""));
			domStyle.set(t.followUpUserNode,			"display", (disabled ? "" : "none"));
			domStyle.set(t.followUpUserFieldContainer,	"display", (disabled ? "none" : ""));
			domStyle.set(t.accountNotesFieldContainer,	"display", (disabled ? "none" : ""));

			domStyle.set(t.replacedClaimContainer, "display", (t.props.value  && t.props.value.claim.replacedClaim != "" ? "" : "none"));
			domStyle.set(t.replacedByClaimContainer, "display", (t.props.value  && t.props.value.claim.replacedByClaims != "" ? "" : "none"));

			t.resize();
			t.inherited(arguments);
		},

		_setValueAttr: function(value) {
			var t = this;
			if (value != null) {
				value.accountStatus	= value.account.status;
				value.dcn			= value.account.dcn;
				value.followUpDate	= value.account.followUpDate;
				value.followUpUser	= value.account.followUpUserId;

				var accountNotes = "";
				arrayUtils.forEach(value.notes, function(accountNote) {
					var noteString = accountNote.date + " - " + accountNote.user + " - " + accountNote.note;
					accountNotes += ((accountNotes == "") ? "" : "<br>") + noteString;
				});
				t.accountNotesNode.innerHTML = (accountNotes !== "" ? accountNotes : "No Claim Notes Found");

				var policyNotes = "";
				arrayUtils.forEach(value.policies, function(policy) {
					if (policy.comments) {
						var noteString = "<label style='font-weight: bold'>" + policy.insurance + " - Effective " + utils.formatters.date("value")({value:policy.effectiveDate}) + "</label><br>" + policy.comments;
						policyNotes += ((policyNotes == "") ? "" : "<br>") + noteString;
					}
				});
				t.policyNotesNode.innerHTML = (policyNotes !== "" ? policyNotes : "No Policy Notes Found");
			}
			t.inherited(arguments, [value]);
		},

		_getValueAttr: function() {
			var t = this;
			var value = t.inherited(arguments);
			return value;
		},

		getSaveData: function() {
			var t = this;
			var value = t.get("value");
			value.id = t.props.value.id;
			return value;
		},

		resize: function() {
			var t = this;
			t.chargeGrid.resize();
			t.patientPoliciesGrid.resize();
			t.submissionGrid.resize();
			t.transactionsGrid.resize();
		}
	});
});