define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"thornberry/app/Component/_Component",
	"thornberry/app/Component/_EditorViewMixin",
	"./AccountEditor",
	"thornberry/app/service/AccountsReceivable",
	"dojo/Deferred",
	"thornberry/app/utils"
],function(
	declare,
	lang,
	arrayUtils,
	_Component,
	_EditorViewMixin,
	AccountEditor,
	service,
	Deferred,
	utils
) {
	return declare([_Component,_EditorViewMixin], {
		title: "Account",

		EditorClass: AccountEditor,

		lockCheckEnabled: true,
		objectType: "Billing.AccountsReceivable.Account",

		objectId: null,
		account: null,

		getTitle: function() {
			if (this.editor.props.value) {
				var patient = this.editor.props.value.patient;
				var claim = this.editor.props.value.claim;
				var description = patient.Name + " (" + patient.AccountId + ")";
				if (claim) {
					description += " - " + claim.typeName + " - " + claim.billPeriodDisplay;
				}
				return (this.props.mode === "view" ? "Account Details" : "Edit Account") + " - " + description;
			} else {
				return this.title;
			}
		},

		getInitialProps: function() {
			return lang.mixin(this.inherited(arguments), {
				mode: "edit", // view or edit
				onSave: null
			});
		},

		getPageProps: function() {
			var t = this;
			var props = lang.mixin(t.inherited(arguments), {
				accountSummaryShown: false
			});
			return props;
		},

		refreshRendering: function() {
			var t = this;
			if (t.isShown) {
				t.set("objectId", t.props.objectId);

				t.editor.set("props", {
					mode: t.props.mode
				});
			}
			t.inherited(arguments);
		},

		_setObjectIdAttr: function(value) {
			var t = this;
			if (value && t.objectId !== value) {
				t.addReadyPromise(service.getAccount({"id": value}).then(function(account) {
					if (t.objectId === value) {
						t.set("account", account);
					}
				}));
			} else if (!value) {
				t.set("account", null);
			}
			t.objectId = value;
		},

		_setAccountAttr: function(value) {
			var t = this;
			t.account = value;
			t.editor.set("props", {
				value: value
			});
		},

		startSave: function() {
			var t = this;
			var deferred = new Deferred();
			/*if (arrayUtils.some(t.editor.chargeGrid.rows(), function(row) {
				return !row.grid.cellWidget.getCellWidget(row.id, "modifiers").modifiersField.isValid();
			})) {
				utils.showMessage("Please correct any invalid fields.");
				deferred.reject();
			} else {*/
				t.inherited(arguments).then(deferred.resolve, deferred.reject);
			//}
			return deferred.promise;
		},

		save: function() {
			var t = this;
			var data = t.editor.getSaveData();
			return service.editAccount(data).then(function() {
				t.objectId = null;
				if (t.props.onSave) {
					t.props.onSave();
				}
			});
		},

		resize: function() {
			var t = this;
			t.editor.resize();
		}
	});
});