define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"thornberry/object/TableManager",
	"thornberry/app/Component/Report/View",
	"dijit/Menu", 
	"dijit/MenuItem", 
	"dijit/form/DropDownButton",
	"thornberry/report/GridView",
	"thornberry/app/Service/AccountsReceivable",
	"thornberry/app/utils"
],function(
	declare,
	lang,
	arrayUtils,
	TableManager,
	ReportView,
	Menu,
	MenuItem,
	DropDownButton,
	GridView,
	service,
	utils
) {
	return declare(ReportView,{
		title: "Accounts",

		defaultToFilterView: false,
		showPresetControl: false,

		controllerProps: {
			createGrid: function() {
				var t = this;
				t.set("gridView",new GridView({
					controller: t,
					title: t.report.displayName
				}));
			}
		},

		getInitialProps: function() {
			return lang.mixin(this.inherited(arguments),{
				reportClass: "Billing.Report.Accounts",
				filters: null
			});
		},

		getPageProps: function() {
			var t = this;
			var selectedIds = t.props.selectedIds;

			var singleSelection = selectedIds.length == 1;
			var isPrePayment = selectedIds.length > 0 && arrayUtils.some(selectedIds,function(id) { return t.getGridData(id)["Account Type"] == "Pre-Payment"; });
			var canMark = (selectedIds.length > 0 && !isPrePayment);
			var canTransfer = (singleSelection && !isPrePayment && (t.getGridData(selectedIds[0]).Balance > 0));

			return lang.mixin(t.inherited(arguments), {
				actions: [
					{ label: "Refresh", onClick: function() {t.controller.gridView.updateGridData(true);} },
					[
						{ label: "View", onClick: function() { t.loadSubView("account", { mode: "view" }); }, hidden: !singleSelection },
						{ label: "Edit", onClick: function() { t.loadSubView("account", { mode: "edit" }); }, hidden: !singleSelection },
						( canMark ? t.markAsButton : null), 
						{ label: "Post Payment", onClick: function() { t.loadSubView("transaction", { mode: "add", transactionType: 2 }); }, hidden: !singleSelection },
						{ label: "Post Adjustment", onClick: function() { t.loadSubView("transaction", { mode: "add", transactionType: 3 }); }, hidden: (!singleSelection || isPrePayment) },
						{ label: "Transfer Balance", onClick: function() { t.loadSubView("balanceTransfer"); }, hidden: !canTransfer },
						{ label: "Prepay", onClick: function() { t.loadSubView("prepayment"); } }
					]
				]
			});
		},

		buildRendering: function() {
			var t = this;
			t.inherited(arguments);
			
			t.markAsButton = new DropDownButton({
				label: "Mark as..."
			});
			t.markAsButton.startup();

			TableManager.get("Tables.ArAccountStatus").then(function(accountStatuses) {
				var menu = new Menu({ style: "display: none;"});
				arrayUtils.forEach(accountStatuses.data, function(item) {
					var menuItem = new MenuItem({
						label: item.name,
						onClick: function() { t.updateAccountStatus(item.id); }
					});
					menu.addChild(menuItem);
				});
				t.markAsButton.set("dropDown", menu);
			});
		},

		postCreate: function() {
			var t = this;
			t.inherited(arguments);

			// Force the report initialization to occur when the module preloads the view
			t.invalidateRendering();
		},

		onShow: function() {
			var t = this;
			t.inherited(arguments);
			if (t.firstShow) {
				t.addSubView("account","thornberry/app/Component/AccountsReceivable/AccountEditorView", function() {
					return {
						objectId: t.props.selectedIds[0],
						onSave: lang.hitch(t,"refreshData")
					};
				});
				t.addSubView("transaction","thornberry/app/Component/AccountsReceivable/TransactionView", function() {
					return {
						objectId: t.props.selectedIds[0],
						onSave: lang.hitch(t,"refreshData")
					};
				} );
				t.addSubView("balanceTransfer","thornberry/app/Component/AccountsReceivable/BalanceTransferView", function() {
					return {
						objectId: t.props.selectedIds[0],
						onSave: lang.hitch(t,"refreshData")
					};
				} );
				t.addSubView("prepayment","thornberry/app/Component/AccountsReceivable/PrePaymentView", function() {
					return { onSave: lang.hitch(t,"refreshData") };
				} );
				
				/*t.addSubView("transaction","thornberry/app/Component/AccountsReceivable/ClaimView",function() {
					return {
						objectId: t.props.selectedIds[0],
						viewOnly: (t.props.selectedIds.length && t.getGridData(t.props.selectedIds[0]).Status === "Pending"),
						onSave: lang.hitch(t,"refreshData")
					};
				});

				t.addSubView("postToAr","thornberry/app/Component/AccountsReceivable/PostToArView",function() {
					return {
						claimIds: t.props.selectedIds,
						onComplete: t.props.onUpdateData
					};
				});*/
			}
		},

		onRowDoubleClick: function(id) {
			console.log(this.getGridData(id));
			this.loadSubView("account", { objectId: id, mode: "view" } );
		},

		updateAccountStatus: function(status) {
			var t = this;
			var promise = service.updateAccountStatus(t.props.selectedIds, status);
			t.props.onSetLoadingPromise(promise);
			promise.then(function() {
				t.props.onUpdateData();
			}, utils.showMessage);
		}
	});
});