define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"thornberry/app/Component/_EditorComponent",
	"thornberry/app/Component/_DataFormMixin",
	"thornberry/app/Component/_TableFieldEditorMixin",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!./Templates/TransactionEditor.html",
	"dojo/text!./Templates/PatientSection.html",
	"dojo/text!./Templates/ClaimSection.html",
	"thornberry/object/TableManager",
	"dojo/dom-style",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/on",
	"dojo/aspect",
	"dojo/when",
	"gridx/Grid",
	"gridx/core/model/cache/Sync",
	"gridx/modules/CellWidget",
	"dojo/store/Memory",
	"thornberry/app/utils",
	"dijit/form/ValidationTextBox",
	"dijit/form/Textarea",
	"dijit/form/CurrencyTextBox",
	"dijit/form/Select",
	"dijit/form/DateTextBox",
	"dijit/form/CheckBox"
],function(
	declare,
	lang,
	arrayUtils,
	_EditorComponent,
	_DataFormMixin,
	_TableFieldEditorMixin,
	_TemplatedMixin,
	_WidgetsInTemplateMixin,
	template,
	patientSectionTemplate,
	claimSectionTemplate,
	TableManager,
	domStyle,
	domClass,
	domConstruct,
	on,
	aspect,
	when,
	Grid,
	Cache,
	CellWidget,
	Memory,
	utils
) {
	return declare([_EditorComponent, _DataFormMixin, _TemplatedMixin, _WidgetsInTemplateMixin, _TableFieldEditorMixin], {
		templateString: template,

		amounts: null,

		postMixInProperties: function() {
			var t = this;
			t.inherited(arguments);

			t.amounts = {};

			t.templateString = lang.replace(template,{
				patientSectionTemplate: patientSectionTemplate,
				claimSectionTemplate:   claimSectionTemplate
			});
		},

		buildRendering: function() {
			var t = this;
			t.inherited(arguments);

			var amountDecorator = function() {
				return "<div><input data-dojo-type='dijit/form/CurrencyTextBox' data-dojo-attach-point='amountField' data-dojo-props=\"selectOnClick: true, constraints: {currency:'USD',min:0.01,max:99999.99}\"><span data-dojo-attach-point='amountNode'></span></div>";
			};
			var amountSetCellValue = function(gridData, storeData, cellWidget) {
				var id = cellWidget.cell.row.id;
				var data = t.chargeGrid.store.get(id);
				var amountField = cellWidget.amountField;
				if (amountField._cnnt) {
					amountField._cnnt.remove();
				}
				if (data.id === "summary"){
					domStyle.set(amountField.domNode, "display", "none");
					domStyle.set(cellWidget.amountNode, "display", "none");
				} else if (t.props.disabled) {
					domStyle.set(amountField.domNode, "display", "none");
					domStyle.set(cellWidget.amountNode, "display", "");
				   
					var node = domConstruct.toDom("<span>"+(data.amount ? data.amount : "")+"</span>");
					domConstruct.place(node, cellWidget.amountNode,"only");
				} else {
					domStyle.set(amountField.domNode, "display", "");
					domStyle.set(cellWidget.amountNode, "display", "none");
					amountField._onChangeActive = false;
					amountField.set("value", data.amount);
					amountField._onChangeActive = true;

					amountField._cnnt = on(amountField, "change", function() {
						t.onAmountChanged(id, amountField.get("value"));
					});
				}
			};
			var chargeStructure = [
				{ id: "type", field: "type", name: "Type" },
				{ id: "description", field: "description", name: "Description" },
				{ id: "chargeDate", field: "chargeDate", name: "Date", formatter: utils.formatters.date("chargeDate") },
				{ id: "chargeCode", field: "chargeCode", name: "Charge Code" },  
				{ id: "net", field: "net", name: "Net", formatter: utils.formatters.dollars("net"), style: "text-align: right" },
				{ id: "balance", field: "balance", name: "Balance", formatter: utils.formatters.dollars("balance"), style: "text-align: right" },
				{ id: "hcpcsCode", field: "hcpcsCode", name: "HCPCS Code" },
				{ id: "amount", field: "amount", name: "Amount", widgetsInCell: true, decorator: amountDecorator, setCellValue: amountSetCellValue, width: "200px" }
			];
			t.chargeGrid = new Grid({
				cacheClass: Cache,
				store: new Memory(),
				structure: chargeStructure,
				autoHeight: true,
				modules: [CellWidget]
			});
			t.chargeGrid.connect(t.chargeGrid.body, "onAfterRow", function(row) {
				var item = row.grid.model.byId(row.id).item;
				try {
					if (item.id === "summary") {
						domClass.add(row.node(),"bold");
					}
				} catch (e) {console.error(e);}
			});
			t.chargeGrid.connect(t.chargeGrid.cellWidget, "onCellWidgetCreated", function(widget, column) {
				// Not all widgets are necessarily created at once; this catches those created
				// when rows scroll into view.
				try {
					t.styleAmountField(widget.amountField);
				} catch (e) {console.error(e);}
			});
			t.chargeGrid.startup();
			t.chargeGridContainer.appendChild(t.chargeGrid.domNode);

			t.addLogic("applyIndividually", function(name, value) {
				t.updateAmountFieldStates();
			}, true);

			var transactionTypeTable = [
				{id: 2, name: "Payment"},
				{id: 3, name: "Adjustment"},
				{id: 8, name: "Apply Pre-Payment"}
			];
			t.setTable("transactionType", transactionTypeTable, ["transactionType"]);
			
			when(TableManager.get("Tables.ArAccountStatus"),function(table) {
				t.accountStatus.setStore(table);
			});

			TableManager.load(["Tables.PaymentType", "Tables.AdjustmentType"]);

			t.addRequirement(["transactionSubType", "transactionDate", "totalAmount"], true);
			t.addDisabled(["totalAmount"], function() {
				return t.getField("applyIndividually").get("checked");
			});
		},

		refreshRendering: function() {
			var t = this;

			if (t.value !== t.props.value) {
				t.value = t.props.value;
				
				t.set("value",t.props.value);

				t.chargeGrid.setStore(new Memory({data:t.props.value ? t.props.value.charges.charges : []}));
				t.chargeGrid.model.clearCache();
				t.chargeGrid.body.refresh();
			} else {
				t.chargeGrid.model.clearCache();
				t.chargeGrid.body.refresh();
			}

			t.updateAmountFieldStates();

			var disabled = t.props.disabled;
			domStyle.set(t.accountStatusNode,			"display", (disabled ? "" : "none"));
			domStyle.set(t.accountStatusFieldContainer,	"display", (disabled ? "none" : ""));
			domStyle.set(t.dcnNode,						"display", (disabled ? "" : "none"));
			domStyle.set(t.dcnFieldContainer,			"display", (disabled ? "none" : ""));
			domStyle.set(t.followUpDateNode,			"display", (disabled ? "" : "none"));
			domStyle.set(t.followUpDateFieldContainer,	"display", (disabled ? "none" : ""));
			domStyle.set(t.followUpUserNode,			"display", (disabled ? "" : "none"));
			domStyle.set(t.followUpUserFieldContainer,	"display", (disabled ? "none" : ""));

			domStyle.set(t.replacedClaimContainer,		"display", (t.props.value  && t.props.value.claim.replacedClaim != "" ? "" : "none"));
			domStyle.set(t.replacedByClaimContainer,	"display", (t.props.value  && t.props.value.claim.replacedByClaims != "" ? "" : "none"));

			t.inherited(arguments);
		},

		updateAmountFieldStates: function() {
			var t = this;
			
			arrayUtils.forEach(t.chargeGrid.rows(), function(row) {
				var cellWidget = row.grid.cellWidget.getCellWidget(row.id, "amount");
				if (cellWidget) {
					t.styleAmountField(cellWidget.amountField);
				}
			});
		},

		styleAmountField: function(amountField) {
			var t = this;
			
			var applyIndividually = t.getField("applyIndividually").get("checked");
			if (!applyIndividually) {
				amountField.set("value", "");
			}

			amountField.set("disabled", !applyIndividually);
			
			var val = amountField.get("value");
			var isNull = (val === "" || /^\s+$/.test(val) || /*NaN check*/ val != val || val === null || typeof val === "undefined" || (typeof val === "object" && val.length === 0));
			var node = (amountField.reqBkgdNode ? amountField.reqBkgdNode : amountField.domNode);
			domClass.toggle(node, "pinkBackground", applyIndividually && isNull);
		},

		updateModified: function(subClassValue) {
			var t = this;
			var modified = false;
			for (var id in t.amounts) {
				modified = true;
				break;
			}
			t.inherited(arguments,[modified]);
		},

		onAmountChanged: function(claimId,value) {
			var t = this;
			var storeData = t.chargeGrid.store.get(claimId);
			if (storeData.amounts == value) {
				delete t.amounts[claimId];
			} else {
				t.amounts[claimId] = value;
			}
			t.styleAmountField(t.chargeGrid.cellWidget.getCellWidget(claimId, "amount").amountField);
			t.invalidate("modified");
		},

		_setValueAttr: function(value) {
			var t = this;
			if (value != null) {
				value.accountStatus		= value.account.status;
				value.dcn				= value.account.dcn;
				value.followUpDate		= value.account.followUpDate;
				value.followUpUser		= value.account.followUpUserId;

				if (typeof value.applyIndividually == "undefined") {
					value.applyIndividually = false;
				}

				value.transactionType = t.props.transactionType.toString().split();
				if (value.transactionType) {
					var transactionType = "", subTypeTable = "";
					if (value.transactionType == 2) {
						transactionType = "Payment";
						subTypeTable = "Tables.PaymentType";
					} else if (this.props.transactionType == 3) {
						transactionType = "Adjustment";
						subTypeTable = "Tables.AdjustmentType";
					}

					when(TableManager.get(subTypeTable), function(table) {
						t.setTable("transactionSubType", table.data, ["transactionSubType"]);
					});

					t.transactionDetailsTitle.innerHTML = transactionType + " Details";
					t.transactionSubTypeLabel.innerHTML = transactionType + " Type:";
					t.transactionDateLabel.innerHTML = transactionType + " Date:";
					t.chargeTitle.innerHTML = transactionType + " Amount";
					t.applyIndividuallyLabel.innerHTML = "Apply " + transactionType.toLowerCase() + " to individual charges?";
				}
			}
			t.inherited(arguments,[value]);
			t.amounts = {};
		},

		getSaveData: function() {
			var t = this;
			var value = t.get("value");
			value.id = t.props.value.id;
			value.amounts = t.amounts;
			value.transactionType = t.props.value.transactionType;
			return value;
		},

		resize: function() {
			var t = this;
			t.chargeGrid.resize();
		}
	});
});